/************************************************************************
  			microeva.h - Copyright micha
*************************************************************************/

#ifndef MICROEVA_H
#define MICROEVA_H

#include "micro.h"

namespace seemicro
{

/**
  * class MicroEva
  * zur Repräsentation einer micro.Eva
  */
struct MicroEva : public Micro
{
    /**
      * Stellung der Arme relativ zu ihren Ansatzpunkten
      * Einheit ??? (°, Pixel, ...)
      */
    float beta[5];

    /**
      * Zeichnet das durch dieses Objekt in einer der Highlevel-Beschreibung
      * repräsentierte Rad in ein Bild
      *
      * @param where
      *        Ausgabebild
      * @param filled
      *        0: Umrisse zeichnen, 1: fuellen
      * @param arm_auch
      *        0: nur den Aussenring zeichnen, 1: Arm auch einzeichnen
      */
/*    void draw(IplImage* where, bool filled=0,
      double useColor=0, bool setzero=1, bool arm_auch=1);

    virtual void draw(IplImage* where, bool filled=0,
      double useColor=0, bool setzero=1)
    {
      draw(where, filled, useColor, setzero, 1);
    };
*/

    virtual Micro* clone()
    {
      return new MicroEva(*this);
    }

};

} // namespace seemicro

#endif // MICROEVA_H

