#ifndef OUTPUT_H
#define OUTPUT_H

#include <list>
#include <iostream>

using namespace std;

namespace seemicro
{

struct OutputModule
{
  virtual void output(bool keyframe)
  {
#ifdef DEBUG
   cerr << "OutputModule::output(bool);\n";
#endif
  };

  void registerMe(std::list<OutputModule*>& l)
  {
#ifdef DEBUG
    cerr << "Registering this...";
#endif
    l.push_back(this);
  }

  virtual ~OutputModule() {};
};

}

#endif
