#ifndef HSVRANGE_H
#define HSVRANGE_H

#include "recognition.h"

namespace seemicro
{

/**
  * Repräsentiert einen Ausschnitt des HSV-Farbraums
  * Dieses Modul ist von RecognitionModule abgeleitet,
  * obwohl es selbst keine Berechnung anstellt, damit
  * bei einer Parameteränderung Nachfolgemodule dies mit
  * Prüfung von outputChanged feststellen können.
  */
struct HSVRange : RecognitionModule
{
  int hmin, hmax, smin, smax, vmin, vmax;

  HSVRange()
  {
    hmin = 40;
    hmax = 110;
    smin = 85;
    smax = 255;
    vmin = 50;
    vmax = 255;
  }

};

/**
  * Gibt eine HSVRange in einen Stream aus.
  */
std::ostream& operator<<(std::ostream& s, const HSVRange& r);

} // seemicro

#endif

