
#ifndef IMAGESOURCE_H
#define IMAGESOURCE_H

#include "recognition.h"
#include <opencv/cv.hpp>

namespace seemicro
{
  struct ImageSource : public RecognitionModule
  {
    /**
      * Image in img is created by this module,
      * other modules can fetch it from here
      */
    IplImage *img;
  };
}

#endif // IMAGESOURCE_H
