#include "hsvrange.h"

namespace seemicro
{

/**
  * Gibt eine HSVRange in einen Stream aus.
  */
std::ostream& operator<<(std::ostream& s, const HSVRange& r)
{ 
  return s << "hmin=" << r.hmin
           << " hmax=" << r.hmax
           << " smin=" << r.smin
           << " smax=" << r.smax
           << " vmin=" << r.vmin
           << " vmax=" << r.vmax;
}

}
